/***************************************************************************
    ExtendedFileFilter - Extends functionality of the FileFilter class

    author:  Ulrich Krebs
    email :  ukrebs@freenet.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

import java.io.File;
import java.util.*;
import javax.swing.*;
import javax.swing.filechooser.*;

class ExtendedFileFilter extends FileFilter
{
	Vector extensionList;
	String description, fullDescription;
	
	public ExtendedFileFilter()
	{
		this("","");
	}

	public ExtendedFileFilter(String extensionList, String description)
	{
		int i;
		StringTokenizer st;
		
		st=new StringTokenizer(extensionList," ,");
				
		this.extensionList=new Vector();
		this.description=description;
		this.fullDescription=this.description;
		
		while(st.hasMoreTokens())
		{
			addExtension(st.nextToken());
		}
	}
		
	public void addExtension(String extension)
	{
		if(extension!=null && extension.length()>0)
		{
			extensionList.addElement(extension.toLowerCase());
			generateFullDescription();
		}
	}
	
	public void setDescription(String description)
	{
		this.description=description;
		generateFullDescription();
	}
	
	private String getExtension(File f)
	{
		if(f!=null)
		{
			String filename=f.getName();
			int i = filename.lastIndexOf('.');
			if(i>0 && i<filename.length()-1)
			{
				return filename.substring(i+1).toLowerCase();
			}
		}
		return null;
	}
	
	private void generateFullDescription()
	{
		int i;
		
		fullDescription=description+" (";
		
		for(i=0;i<extensionList.size();i++)
		{
			fullDescription+=(String)extensionList.elementAt(i);
			if(i<extensionList.size()-1) fullDescription+=", ";
		}
		fullDescription+=")";
	}
	
	
	public boolean accept(File f)
	{
		int i;
		
		if(f!=null)
		{
			if(f.isDirectory())
			{
				return true;
			}
			String extension=getExtension(f);
			if(extension!=null && extensionList.contains("*."+extension) || extension==null && extensionList.contains("*"))
			{
				return(true);
			}
		}
		return(false);
	}

	public String getDescription()
	{
		return(fullDescription);
	}
}